/*= main_option1.c =============================================================
 *
 * Copyright (C) 2005 Nordic Semiconductor
 *
 * This file is distributed in the hope that it will be useful, but WITHOUT
 * WARRANTY OF ANY KIND.
 *
 * Author(s): Borge Strand
 *
 * Description:
 *
 *    Startup, address generation, user interface for user interface option 1
 *    with soft power on only at ATX 
 *
 * Compiler: Tested with WinAVR, avr-gcc (GCC) 3.4.3
 *
 * Revision: 2.0 
 *
 *==============================================================================
 */


// This program sets up a private address for ARX and ATX, puts the system into sleep mode, and
// runs the user interface. After batteries are inserted and the on/off button is flipped to "ON", 
// the user must press and hold the ATX Play button for music to appear. The time the Play button
// has to be held depends on the aggressiveness of the time-division-multiplexing used in sleep mode;
// lower current consumption trades against the Play button having to be held for a longer time in
// order to turn on the system.
// This program receives all compatible buttons possible from both ATX and ARX user interfaces.
// Fill in here how different button commands are to be passed to the audio source system. This 
// program only executes DAC volume control and nRF24Z1 power up/down.

// Certain parameters in z1config must be changed to work with this file. After restoring factory 
// default settings, please make the following alterations for user interface option 1:
//
// Link:
// LNKETH=0x20
// LNKMOD=0x08, Mute when LNKERRLNKETH
// NLCH=0x0F
// ADDR, use your preferred initial address. This address must match that of function z1_setinitialadr().
//
// Interrupts:
// INTCF=0x02, Enable wakeup from power down interrupt
//
// ATX:                                             ARX:
// I2SCNF_IN=0x80, Audio mode = Master mode         RXSTA=0x40, Disable serial slave interface
// TXLAT=0x04, Nominal, 20ms @ 48kHz                RXWTI=0x01
// TXWTI=0x01                                       RXLTI=0x31
// TXLTI=0x31                                       RXSTI=0x0030
// TXSTI=0x0000                                     RXWAKE=0x10, Wakeup on sleep timer
// TXMOD=0xE2, RF, I2S, TX pwr down, 44.1, wake@DD  I2SCNF_OUT=0x40, Mute sound
//                                                  RXMOD=0x20, RF, I2S

#define SLEEP_TXWTI     0x01                        // ATX may go to auto power down
#define SLEEP_TXSTI_0   0x00                        // ATX sleep timer is not used, ATX is wake-on-interrupt
#define SLEEP_TXSTI_1   0x00
#define SLEEP_TXLTI     0x31                        // ATX powers down after 0x01 * (0x31 + 1) * 10ms = 500ms without link

#define SLEEP_RXWTI     0x01                        // ARX is awake for 0x01 * 10ms = 10ms, ARX may go to auto power down
#define SLEEP_RXSTI_0   0x30                        // ARX is asleep for 0x30 * 10ms = 480ms
#define SLEEP_RXSTI_1   0x00
#define SLEEP_RXLTI     0x31                        // Auto power down after 1 * (0x31 + 1) * 10ms = 500ms without link
#define SLEEP_RXWAKE    0x10                        // ARX wakes up from Play button, see button definitions in hpref_defines.h


int main(void) {
    char booterror = 0;                             // Wheter or not a certain while loop is to execute, 0 means successfull complesion of bootup step
    char debounce = 1;                              // Check buttons before accepting them
    char buttons;                                   // Which button is pressed?
    char powermode;                                 // The power up/down status of the nRF24Z1
    char muted = 0;                                 // Simulate Play/Pause button by muting and unmuting audio while retaining audio link

    #ifdef USELED
        char ledtemp, ledstatus = LEDON;            // Two registers used to track blink sequences of LEDs
    
        ledsequence = LED_INITIAL;                  // Blink according to linkup pattern
        mcu_atxled(z1_rotate_led());                // Left-rotate the led sequence and apply ATX light according to MSB
    #endif

    mcu_init();                                     // Initialize MCU
    mcu_2w_master_init();                           // Enable 2-wire slave interface to nRF24Z1

    #ifdef DEBUG                                    // If debug system is enabled, 
        mcu_uart_init();                            // Turn on UART for debug
        db_putenter();
        mcu_putchar('-');                           // Print welcome message
        mcu_putchar('D');
        mcu_putchar('E');
        mcu_putchar('V');
        mcu_putchar('-');
        db_putenter();
    #endif
    #ifdef DEBUGIF
        db_hex();                                   // Run debug interface if that is defined
    #endif

    volume      = 0xFF;                             // Volume is not initiated, dac_init() gives default value
    bass        = 0xFF;                             // Bass boost is not initiated, dac_init() gives default value, not used in HP ref design

    // At this stage ATX is just turned on. ARX may be on or off. In any case, do not make any attempt at disabling the ARX DAC. 
    // Instead disable ATX resources (including ADC) and start looking for a link. 

    adc_init();                                     // Headphone Reference Design ADC needs init routine
    adc_sleep();                                    // ADC should not consume power while trying to establish link

    // ATX sleeps with wake-on-interrupt. Poll ATX user interface for Play button, use that to interrupt=wake up ATX nRF24Z1
    // ATX was set in EEPROM to go straight to sleep mode. The sleep enable bit in RXMOD therefore has to be reset before 
	// attempts are made to establish a link. But ARX was set up in EEPROM to go to auto power down. There is therefore 
	// no need to reset its sleep enable bit in RXMOD.
    
    while (booterror == 0) {
        #ifdef USELED
            mcu_atxled(z1_rotate_led());            // Left-rotate the led sequence and apply ATX light according to MSB
        #endif

        buttons = mcu_buttons();                    // Read buttons on local MCU GPIO pins
        if (buttons == BTN_PLAY) {                  // If Play button was pressed, wake up system by:
            mcu_z1wakeup_pin();                     // Toggle the pin that wakes up ATX nRF24Z1 from wake-on-interrupt
            z1_singlewrite(TXWTI, 0x00);            // Disable auto power down, ARX went to auto power down, no need to toggle RXMOD
            z1_singlewrite(TXMOD, z1_singleread(TXMOD) & ~0x40);    // Wake up ATX, keep value of wakeup from DD[1] bit set
            if (z1_haslink_wait()) {                // Did linkup succeede after multiple pollings?
                booterror = 1;                      // If it did, terminate the link searching loop, this does not indicate bootup error!
            }
            else {                                  // If it didn't,
                z1_singlewrite(TXWTI, SLEEP_TXWTI); // Reset TXWTI, ATX will go back to sleep again
                z1_singlewrite(TXMOD, z1_singleread(TXMOD) | 0x40); // Put ATX back to sleep again
            }
        }
        if (booterror == 0)                         // Poll again if: 1) no button was pressed; or if 2) Play was pressed with no link resulting
            mcu_wait_ms(160);                       // Wait and poll button again
    }

    booterror = 0;                                  // Loop ended, prepare to continue bootup process
    
    // A link now exists between ATX and ARX on the initial address. The next thing to do is pick a random private address and
    // put the DAC into the correct power mode. For user interface optio 1, we got to this stage through user interaction,
    // and the DAC should be powered on. Also, the ARX nRF24Z1 should stop muting its digital audio output.
    
    // This code assumes that all units are produced equally and therefore have the same initial address loaded from EEPROM. The next
    // section of the code will pick a new address and transfer that to the ARX. The link will then be re-established with the new
    // private address. If, instead, your products are made in individual pairs, this section may be commented out. Picking a random
    // private address is done with the I2S bitclock as a randomness source. This means that the audio clock and the MCU clock must be 
    // asynchronous. It also means that the I2S interface must be operational. In the headphone reference design, nRF24Z1 ATX is I2S 
    // master. There is therefore no need to start up the ADC in order to generate the bitclock from the master clock.
    
    // With units powered up, change from initial to private address

    if (z1_flagready(LINKCSTATE) == OKAY) {         // If ATX is ready bo buffer private address
        z1_setprivateadr();                         // Draw a new private address, write it to ATX, Important: I2S bitclock must be active now!!
        z1_setflag(LINKCSTATE);                     // Tell ATX to relink with privately connected ARX
        mcu_wait_ms(32);                            // Give it some time to relink
    }
    else {                                          // LINKCSTATE was not ready to transfer a new address. Halt bootup process
        booterror = 1;
    }

    if (booterror == 0) {
        if (z1_haslink() == 0) {
            booterror = 1;
        }
    }

    // Done setting up the private address. Now that a link was established, set up the ARX DAC

    if (booterror == 0) {                           // If no error detected, 
        if (dac_init() != OKAY) {                   // Try to initialize DAC
            booterror = 1;                          // If that failed, stop bootup code
        }
    }

    // With full user interface on ATX, user interaction was needed to bring the system up. Therefore: power up
    // all units and run the user interface. 

    // Insert code to start music on audio source here!

    adc_wake();                                     // Wake up ATX resources 

    if (booterror == 0) {                           // If no error detected, 
        if (dac_wake() != OKAY) {                   // Try to wake up DAC
            booterror = 1;                          // If that failed, stop bootup code
        }
    }

    if (booterror == 0) {                           // If no error detected, 
        if (z1_arx_mute(MUTEOFF) != OKAY) {         // Try to unmute ARX
            muted = 0;                              // Simulate play/pause button
            booterror = 1;                          // If that failed, stop bootup code
        }
    }
    
    // After starting up all subcircuits, the devices are on. This is indicated to the CPU as powermode == PMODE_AWAKE. 
    // In this mode, the sleep enable bits in RXMOD and TXMOD are cleared. Also, TXWTI and RXWTI are set to != 0x00 to
    // enable automatic power down. In RXMOD, the sleep enable bit was set in EEPROM (see top of file) and never cleared 
    // in the program. But the power down bit in TXMOD was set in EEPROM and set upon exiting the link finding loop
    // above. However, TXWTI was never reset to enable auto power down, so this must be done now:
    
    z1_singlewrite(TXWTI, SLEEP_TXWTI);             // Reset TXWTI, ATX auto power down is enabled in PMODE_AWAKE

    #ifdef USELED
        ledsequence = LED_AWAKE;                    // Blink in a pattern indicating audio streaming 
    #endif

    powermode = PMODE_AWAKE;                        // And indicate a power up of ATX


    // Run product interface by polling buttons on TX and RX unit

    while (booterror == 0) {                            
        // Buttons on ARX are hardwired to GPI pins DI[0:3] or written into RXPIN on ARX according to the table 
        // in hpref_defines.h. Buttons on ATX are translated in mcu_xxxx.c from the actual hardware I/O pins on the MCU
        // to a key code equal to the hardware key codes defined for ARX. The key codes are then processed in 
        // this file. All buttons on DI[0:3] are active high. Key codes comprising more than one high bit must
        // be generated by an ARX MCU or diode network. Unused ARX DI[0:3] pins must be tied to ground.
        // This part of the MCU program uses key polling and a low-power wait state. 
    
        if (powermode == PMODE_AUTODOWN)            // Auto power down was detected. Poll interrupt bit for power up
            mcu_wait_ms(320);
        else if (powermode == PMODE_POWERDOWN)      // Controlled power down, MCU timer is used to poll Play button on
            mcu_wait_ms(320);                       // MCU. If Play is pressed, the MCU brings the system out of power down
        if (powermode == PMODE_AWAKE)               // Audio streaming mode, MCU polls for input from user interface 
            mcu_wait_ms(64);

        // There are six combinations of variables powermode and wakeup. These six combinations are summarized below.
        
        //// USER INTERFACE OPTION 1 //// ATX is wake-on-interrupt, ARX is wake-on-timer
        //              |   powermode ==        |   powermode ==            |   powermode == 
        //              |   PMODE_AUTODOWN      |   PMODE_POWERDOWN         |   PMODE_AWAKE
        //              |                       |                           |
        // wakeup ==    |   Check if Play       |   No ATX interrupts       |   ATX interrupts
        // WAKE_Z1_INT  |   button is pressed   |   this state makes        |   this state makes
        //              |   on ATX. If it is,   |   no sense                |   no sense
        //              |   try to set up link  |                           |
        // wakeup ==    |   w/semi-fast polling |   Check Play button to    |   Audio streaming mode,
        // WAKE_WAIT    |   in MCU              |   exit sleep mode         |   MCU polls user interfaces

        if (powermode == PMODE_AUTODOWN) {
            // It was detected that nRF24Z1 went to auto power down. Only the MCU timer will wake up the MCU in user 
            // interface option 1. When going to auto power down, both TXMOD anr RXMOD were set for audio streaming. 
            // Therefore there is no need to clear the sleep enable bits in TXMOD and RXMOD when exiting auto power down. 

            // ATX is wake-on-interrupt, check if user is pressing Play on ATX
            buttons = mcu_buttons();                // Read buttons on local MCU GPIO pins
            if (buttons == BTN_PLAY) {              // If Play button was pressed, wake up system by:
                mcu_z1wakeup_pin();                 // Toggle the pin that wakes up ATX nRF24Z1 from wake-on-interrupt, ATX then awake TXWTI(TXLTI+1)10ms
                if (z1_haslink_wait()) {            // Check for link a number of times, if success:

                    // Instruct your audio source to commence playing music here!

                    adc_wake();                     // Wake up ATX resources
                    dac_wake();                     // Wake up ARX resources, may also test that logical return of dac_wake() indicated success
 
                    z1_arx_mute(MUTEOFF);           // If ARX was muted at by pause button remove muting each time units exit sleep mode by UI
                    muted = 0;                      // Simulating Play/Pause functionality
                    #ifdef USELED
                        ledsequence = LED_AWAKE;    // Blink in a pattern indicating audio streaming 
                    #endif
                    
                    powermode = PMODE_AWAKE;        // And indicate a power up of ATX
                }
            }
            
            #ifdef USELED
                mcu_atxled(z1_rotate_led());        // Alter ATX LED according to blinking sequence, after potentially critical operations!
            #endif

        } // powermode == PMODE_AUTODOWN

        else if (powermode == PMODE_POWERDOWN) {

            if (wakeup == WAKE_Z1_INT) {
                // In user interface option 1 the ATX nRF24Z1 does not generate any interrupts to the MCU while in sleep mode. 
                // That is because it sleeps until it receives an interrupt from the MCU. We are not supposed to receive 
                // any interrupts from sleeping ATX nRF24Z1. Exception is when ARX nRF24Z1 has just received a wakeup interrupt 
                // from the MCU. 

                #ifdef TERMINATE_ON_ERROR
                    booterror = 1;
                #endif
            } // PMODE_POWERDOWN / WAKE_Z1_INT
            
            else if (wakeup == WAKE_WAIT) {
                // In user interface option 1 the ATX is wake-on-interrupt (it wakes on an interrupt 
                // generated by the MCU). It is therefore not able to interrupt the MCU on its own, and the MCU must
                // instead use polling (i.e. its internal timer is used for polling, not as a safety measure). 

                // Poll ATX Play button and bring system out of sleep mode if it is being pressed
                buttons = mcu_buttons();            // Read buttons from local MCU only
                if (buttons == BTN_PLAY) {          // Play button will wake system
                    mcu_z1wakeup_pin();             // Toggle the pin that wakes up ATX nRF24Z1 from wake-on-interrupt
                    z1_singlewrite(TXWTI, 0x00);    // Disable auto power down while sending wakeup commands to ATX
                    z1_singlewrite(TXMOD, z1_singleread(TXMOD) & ~0x40);    // Wake up ATX
                    z1_singlewrite(RXMOD, z1_singleread(RXMOD) & ~0x80);    // Wake up ARX
                    
                    // In user interface option 1, the ARX is wake-on-timer operating on a slow clock. If the user presses the ATX
                    // Play button while the ARX is out of range, no link is established, and the system should go back to the sleep
                    // mode it was in, PMODE_POWERDOWN. 
                    
                    if (z1_haslink_wait()) {        // Wait until a link is established or timeout
                    
                        // Instruct source to commence playing music
                        
                        adc_wake();                 // Wake up ATX resources
                        dac_wake();                 // Wake up ARX resources, may also test that logical return of dac_wake() indicated success
						
                        z1_arx_mute(MUTEOFF);       // If ARX was muted at by pause button remove muting each time units exit sleep mode by UI
                        muted = 0;                  // Simulating Play/Pause functionality
                        #ifdef USELED
                            ledsequence = LED_AWAKE;// Blink in a pattern indicating audio streaming 
                        #endif

                        powermode = PMODE_AWAKE;    // Indicate that system is awake
                        debounce = 1;               // Activate software debounce

                        z1_singlewrite(TXWTI, SLEEP_TXWTI);     // Re-enable auto power down = allow for proper TXLTI timing
                    }
                    else {                          // No link found, power things down again, this is same sequence as Stop button performs
                        z1_singlewrite(RXMOD, z1_singleread(RXMOD) | 0x80);     // Put ARX to sleep
                        z1_singlewrite(TXWTI, SLEEP_TXWTI);         // Re-enable auto power down = allow for proper TXLTI timing
                        z1_singlewrite(TXMOD, z1_singleread(TXMOD) | 0x40); // Put ATX to sleep
                    } // No link found
                } // BTN_PLAY

                #ifdef USELED
                    mcu_atxled(z1_rotate_led());    // Alter ATX LED according to blinking sequence, after potentially critical operations!
                #endif

            } // PMODE_POWERDOWN / WAKE_WAIT
            
        } 
        else if (powermode == PMODE_AWAKE) {
        
            if (wakeup == WAKE_Z1_INT) {
                // In user interface option 1 the ATX nRF24Z1 will not start generating interrupts to the MCU if it goes to 
                // auto power down mode. Instead, its IRQ output will be static at "1" (because the interrupt to the MCU is active low).
                // Because of this, the combination powermode == PMODE_AWAKE, wakeup == WAKE_Z1_INT indicates some kind of error
                // in user interface option 1. 

                // There must be an error, the ARX nRF24Z1 is not supposed to give interrupts to the MCU except when it wakes
                // up after receiving a wakeup interrupt from the MCU.

                #ifdef TERMINATE_ON_ERROR
                    booterror = 1;
                #endif
            } // PMODE_AWAKE / WAKE_Z1_INT

            else if (wakeup == WAKE_WAIT) {
                // This is audio streaming mode. The MCU is polling ARX buttons and local buttons while awake.
                // All buttons are interpreted. For user interface option 1, this section first has to check if the 
                // ATX nRF24Z1 went to automatic power down. That is done by testing the ATX nRF24Z1 output interrupt;
                // if the interrupt pin went high (active low), the ATX shut itself down and went to automatic power down.

                // Check if ATX nRF24Z1 went to automatic power down. If it did, shut down ATX resources like the ADC. 
                // Unfortunately, there is no way to reach the DAC because the link is down. 
                
                if (!mcu_z1int_active()) {          // If ATX nRF24Z1 is no longer active,

                    // Instruct audio source to stop playing music here!

                    adc_sleep();                    // Then, power down ATX resources
                    #ifdef USELED
                        ledsequence = LED_AUTODOWN; // Blink in a pattern indicating auto power down
                    #endif

                    powermode = PMODE_AUTODOWN;     // Indicate that sleep is due to an automatic power down
                }

                if (powermode == PMODE_AWAKE) {     // If option 1 didn't detect an auto power down, process rest of user interface
                    buttons = BTN_NOKEY;            // Assume that no button is pressed
                    if (mcu_z1int_active())         // Is ATX nRF24Z1 awake and listening on the slave interface?           
                        buttons = z1_singleread(RXPIN) & 0x0F;  // Read buttons from ARX
                    if (buttons == BTN_NOKEY)       // If nothing pressed on ARX,
                        buttons = mcu_buttons();    // Process buttons from ATX

                    if (debounce == 0) {                

						// The following buttons are single-action (i.e. require that no button is pressed before requested action is taken)

                        switch (buttons) {
                            case BTN_STOP:
                                // Enter your application code to handle a stop button!

                                dac_sleep();        // First power down DAC before powering down radio ling
                                adc_sleep();        // Power down resources on ATX

                                // TXMOD and RXMOD must now be set so that the two devices do not wake up even if they are within 
                                // range and in link mode at the same time. Such a condition would otherwise have woken them up
                                // again. Wakeup must only happen when the user pressed the Play button, in which case RXMOD and 
                                // TXMOD are set for power on in order to attempt linkup.

                                #ifdef USELED 
                                    z1_arxled(LEDOFF);              // Turn off ARX LED before going to sleep and removing link
                                    ledsequence = LED_POWERDOWN;    // Blink in a pattern indicating power down
                                #endif

                                powermode = PMODE_POWERDOWN;
                                z1_singlewrite(RXMOD, z1_singleread(RXMOD) | 0x80);     // Put ARX to sleep
                                z1_singlewrite(TXWTI, SLEEP_TXWTI);         // Re-enable auto power down = allow for proper TXLTI timing
                                z1_singlewrite(TXMOD, z1_singleread(TXMOD) | 0x40); // Put ATX to sleep

                                mcu_wait_ms(3200);  // Don't actually sleep for 3.2 seconds but pick up the first (stray) interrupt from ATX
                            break;
                            case BTN_PLAY:          // Play button detected
                                // Enter your application code to handle a play/pause button. 
								
								// A pause button simulation is inserted below.
                                // The pause functionality here does not communicate with the audio source. It does not put any unit
                                // into power save mode. It is just to simulate a common look-and-feel

                                if (muted == 0) {           // Simulating Play/Pause functionality
                                    z1_arx_mute(MUTEON);    // If no mute, turn on mute = Pause
                                    muted = 1;
                                }
                                else {                      // If muted, turn off mute = (de)Pause
                                    z1_arx_mute(MUTEOFF);
                                    muted = 0;
                                }
                            break;
                            case BTN_SKIPF:         // Skip forward button detected
                                // Enter your application code to handle a forward skip button
                            break;
                            case BTN_SKIPB:         // Skip backward button detected
                                // Enter your application code to handle a backward skip button
                            break;
                            case BTN_BBOOST:        // Play detected
                                // Enter your application code to handle a bass boost button (button not implemented in Headphone Reference Design)
                            break;
                            default:                // Do not consider other buttons here
                            break;
                        } // end of switch
                    } // debounce == 0

                    // The following buttons may be held to cause repeated action

                   switch (buttons) {
                        case BTN_VOLUP:             // Volume up button detected
                            dac_volume_up();        // This program increases volume in remote DAC
                        break;
                        case BTN_VOLDN:             // Volume down button detected
                            dac_volume_down();      // This program decreases volume in remote DAC
                        break;
                        case BTN_SCANF:             // Scan forward button detected
                            // Enter your application code to handle a forward scan button (button not implemented in Headphone Reference Design)
                        break;
                        case BTN_SCANB:             // Scan backward button detected
                            // Enter your application code to handle a backward scan button (button not implemented in Headphone Reference Design)
                        break;
                        default:                    // Do not consider other buttons here
                        break;
                   } // end of switch
  
                    // Other buttons may be defined here according to your application

                    if (buttons == BTN_NOKEY)           // If no buttons were detected,
                        debounce = 0;                   // Indicate all buttons were released
                    else                                // Some button detected
                        debounce = 1;                   // Activate software debounce
                } // PMODE_AWAKE, not auto power down in user interface option 1

                #ifdef USELED                           // ATX button polling is called every 64ms. That is an okay period for LED blinking
                    if (powermode == PMODE_AWAKE) {     // If no power down etc has been initiated yet, process LED blinking
                        if (mcu_z1int_active()) {       // If ATX nRF24Z1 is active, (important to check in case of auto power down)
                            ledtemp = z1_rotate_led();  // Record present state in LED blinking sequence
                            if (ledstatus != ledtemp) {
                                mcu_atxled(ledtemp);    // Alter ATX LED according to blinking sequence
                                z1_arxled(ledtemp);     // Alter ARX LED according to blinking sequence
                                ledstatus = ledtemp;
                            }
                        }
                    }
                #endif

            } // PMODE_AWAKE / WAKE_WAIT
        }
    } // while (booterror == 0)

    // We arrive here with booterror != 0. Bootup or user interface failed. Program terminates.

    adc_sleep();                                    // Power down resources on ATX

    z1_singlewrite(TXWTI, 0x01);                    // Disable any ATX wake-on-timer
    z1_singlewrite(TXLTI, 0x31);                    // Disable any ATX wake-on-timer
    z1_singlewrite(TXSTI_0, 0x00);                  // Disable any ATX wake-on-timer
    z1_singlewrite(TXSTI_1, 0x00);                  // Disable any ATX wake-on-timer
    z1_singlewrite(TXMOD, 0x62);                    // Minimize ATX power consumption: no RF, TX power down, wake-on-interrupt

    #ifdef USELED
        ledsequence = LED_FAILURE;                  // Blink in a fast on-off-on-off-on-off pattern 
        while (1) {
            mcu_wait_ms(64);                        // Use 4 * 64ms blink period, 64ms blink on time
            mcu_atxled(z1_rotate_led());            // Left-rotate the led sequence and apply ATX light according to MSB
        }
    #else
        while (1) {                                 // Terminate in infinite loop
        }       
    #endif

    return 0;                                       // Insert a return statement despite infinite loop

} // int main()
